#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "bufferwrapper4xi.py">
#<abstract4xi abstract = "provides class BufferWrapper4Xi"/>
#<refer4xi reference = "heap.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------
#
# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'BufferWrapper4Xi' ]
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass    
try:                 # most-common ones
    import tempfile
    import itertools         
except:
    pass
try:                 # rare ones
    import bisect
    import mmap       
except:
    pass 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    print("bufferwrapper4xi.py: *** failed in import _multiprocessing ***")
try:                             # Source ones
    from multiprocessing import heap
except:
    print("bufferwrapper4xi.py: *** failed in from multiprocessing import heap ***")
try:                             # Source ones
    from multiprocessing.heap import BufferWrapper
except:
    print("bufferwrapper4xi.py: *** failed in from multiprocessing.heap import BufferWrapper ***")          
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from examine4xi import examine4xi
from finalize4xi import Finalize4Xi 
from heap4xi import Heap4Xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from print4xi import print4xi
from time4xi import time4xi
#"
#/>

# =================================================================================================
#<class4xi class = "BufferWrapper4Xi" base = "object" child = "None" synopsis = "Representing a chunk of an mmap" usage = "
#   sharedctypes.py:
#       wrapper = heap.BufferWrapper(size)
#   _new_value4xi.py:
#       wrapper2xi = BufferWrapper4Xi(size2xi)
#   test_finalize4xi.py
#       b2xi = BufferWrapper4Xi(4096)
#       block2xi = b2xi._heap.malloc(4096)
#   barrier4xi.py:
#       wrapper2xi = BufferWrapper4Xi(struct.calcsize('i') * 2)
#   heap4xi.py:
#       block2xi = BufferWrapper4Xi._heap.malloc(size)
#> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
#
class BufferWrapper4Xi(object):
    '''
    BufferWrapper4Xi class represents a chunk of an mmap
    '''
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #expose4xi exposure = "
    _exposed_ = ('get_address','get_size','create_memoryview','sense', 'reason', 'ace')          
    #"
    #/>

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    _heap = Heap4Xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-BufferWrapper4Xi" usage = "initializes BufferWrapper4Xi">
    # ---------------------------------------------------------------------------------------------
    def __init__(self, size):
        '''
        BufferWrapper4Xi __init__ methods initializes a BufferWrapper4Xi object
        ''' 
        # -----------------------------------------------------------------------------------------    
        # Step 1/5 
        # ----------------------------------------------------------------------------------------- 
        class2print = 'BufferWrapper4Xi' 
        method2print = '__init__'
        
        message2print = "{(Step 1/5a) started" + "; init-of-BufferWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        # 18:40:44 02/22/13 Pacific Standard Time
         
        message2print = "{(Step 1/5b) borning" + "; bufferwrapper2xi-of-BufferWrapper4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 18:40:44 02/22/13 Pacific Standard Time
        
        message2print = "{(Step 1/5c) inheriting" + "; object-inheriting-by-bufferwrapper2xi-of-BufferWrapper4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'object'>
        
        message2print = "{(Step 1/5d) sharing" + "; heap-sharing-by-bufferwrapper2xi-of-BufferWrapper4Xi: " + str(BufferWrapper4Xi._heap)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
        # <heap4xi.Heap4Xi object at 0x0000000002AF16D8>
                                
        message2print = "{(Step 1/5e) passed; size-passed-to-init-of-BufferWrapper4Xi: " + str(size) + ";time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 12; 4096
        
        # -----------------------------------------------------------------------------------------    
        # (Step 2/5) - ASSERT SIZE
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 2/5a) got; sys-maxsize-got-by-init-of-BufferWrapper4Xi: " + str(sys.maxsize)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 9223372036854775807
        
        assert 0 <= size < sys.maxsize
        
        if examine4xi(class2print, method2print):  
        
            # -------------------------------------------------------------------------------------    
            # StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) performing" + "; BufferWrapper-init-performing-by-init-of-BufferWrapper4Xi: " + str(BufferWrapper.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                
                             
            BufferWrapper.__init__(self, size)
            
            message2print = "{(StepOriginal) performed" + "; BufferWrapper-init-performed-by-init-of-BufferWrapper4Xi: " + str(BufferWrapper.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                        
        else:            
        
            # -----------------------------------------------------------------------------------------    
            # (Step 3/5)
            # -----------------------------------------------------------------------------------------             
            try: 
                block2xi = BufferWrapper4Xi._heap.malloc(size)              # uses class variable _heap and its method malloc
    
                            
                message2print = "{(Step 3/5a) set; block2xi-set-by-init-of-BufferWrapper4Xi to: " + str(block2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # (<arena4xi.Arena4Xi object at 0x00000000035ED080>, 0, 4096)
                # (<arena4xi.Arena4Xi object at 0x0000000003A84F60>, 0, 8)           
            except:
                message2print = "{(Step 3/5b) hit; exception-hit-by-init-of-BufferWrapper4Xi: *** failed to set block2xi ** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #            
            
                raise
                
            self._state = (block2xi, size)
            
            message2print = "{(Step 3/5c) set; state-of-bufferwrapper2xi-set-by-init-of-BufferWrapper4Xi to: " + str(self._state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # ((<arena4xi.Arena4Xi object at 0x00000000035ED080>, 0, 4096), 4096)
            
            
            # -----------------------------------------------------------------------------------------    
            # (Step 4/5)
            # -----------------------------------------------------------------------------------------        
            message2print = "{(Step 4/5a) passing; block2xi-passing-to-Finalize4Xi-by-init-of-BufferWrapper4Xi: " + str(block2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # (<arena4xi.Arena4Xi object at 0x00000000038457B8>, 0, 4096)
            
    
            try:
                Finalize4Xi(self, BufferWrapper4Xi._heap.free, args=(block2xi,))          
                
                message2print = "{(Step 4/5c) performed; Finalize4Xi-performed-by-init-of-BufferWrapper4Xi: " + str(Finalize4Xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <class 'finalize4xi.Finalize4Xi'>
            except:
                message2print = "{(Step 4/5z) failed; Finalize4Xi-failed-by-init-of-BufferWrapper4Xi: Finalize4Xi with block2xi: " + str(Finalize4Xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
    
            finally:                
                # -----------------------------------------------------------------------------------------    
                # (Step 5/5)
                # -----------------------------------------------------------------------------------------        
                message2print = "{(Step 5/5a) inherited" + "; object-inherited-by-bufferwrapper2xi-of-BufferWrapper4Xi: " + str(object)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # <class 'object'>
                   
                message2print = "{(Step 5/5b) activated" + "; bufferwrapper2xi-of-BufferWrapper4Xi-activated: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 19:23:24 02/22/13 Pacific Standard Time
                
                message2print = "{(Step 5/5c) shared" + "; heap-shared-by-bufferwrapper2xi-of-BufferWrapper4Xi: " + str(BufferWrapper4Xi._heap)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                      
                # <heap4xi.Heap4Xi object at 0x0000000002BA7F60>
                  
                message2print = "{(Step 5/5d) returning" + "; init-of-BufferWrapper4Xi-returning: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                # 19:23:24 02/22/13 Pacific Standard Time
             
    #</methodize4xi>   
          
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get-address-of-BufferWrapper4Xi" usage = "
    #     sharedctypes.py:
    #         wrapper = heap.BufferWrapper(size)
    #         obj = type_.from_address(wrapper.get_address())
    #">
    # ---------------------------------------------------------------------------------------------
    def get_address(self): 
        '''
        BufferWrapper4Xi method get_address returns address
        '''
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'BufferWrapper4Xi'
        method2print = 'get_address'
        
        message2print = "{started" + "; get-address-of-BufferWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        # 05:26:54 04/15/13 Pacific Daylight Time
        
        if examine4xi(class2print, method2print):  
        
            # -------------------------------------------------------------------------------------    
            # StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) performing" + "; BufferWrapper-get_address-performing-by-init-of-BufferWrapper4Xi: " + str(BufferWrapper.get_address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                
                             
            return BufferWrapper.get_address(self)
            
        else:            
           
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------         
            (arena2xi, start2xi, stop2xi), size2xi = self._state
            
            
            message2print = "{set" + "; arena2xi-set-by-get-address-of-BufferWrapper4Xi to: " + str(arena2xi)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)            
            # <arena4xi.Arena4Xi object at 0x00000000039F1FD0>
            
            message2print = "{set" + "; start2xi-set-by-get-address-of-BufferWrapper4Xi to: " + str(start2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 0  
            
            message2print = "{set" + "; stop2xi-set-by-get-address-of-BufferWrapper4Xi to: " + str(stop2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 8
            
            message2print = "{set" + "; size2xi-set-by-get-address-of-BufferWrapper4Xi to: " + str(size2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 4
                  
            # -----------------------------------------------------------------------------------------    
            # JATKA - SOMETIMES TILTS HERE!
            # -----------------------------------------------------------------------------------------         
            address2xi, length2xi = _multiprocessing.address_of_buffer(arena2xi.buffer) # to be analyzed! 
            
            message2print = "{set" + "; address2xi-set-by-get-address-of-BufferWrapper4Xi to: " + str(address2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{set" + "; length2xi-set-by-get-address-of-BufferWrapper4Xi to: " + str(length2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
               
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------                 
            assert size2xi <= length2xi
            
            message2print = "{asserted" + "; size2xi-asserted-by-address-of-BufferWrapper4Xi: " + str(size2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)     
            
            message2print = "{asserted" + "; length2xi-asserted-by-address-of-BufferWrapper4Xi: " + str(length2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)     
                
            # -----------------------------------------------------------------------------------------    
            #
            # ----------------------------------------------------------------------------------------- 
            message2print = "{returning" + "; address2xi-and-start2xi-returning-by-get-address-of-BufferWrapper4Xi: " + str(address2xi + start2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)                    
            
            return address2xi + start2xi
       
    #</methodize4xi>  

          
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create-memoryview-of-BufferWrapper4Xi" usage = "
    #     sharedctypes.py:
    #         rebuild_ctype
    #             buf = wrapper.create_memoryview()
    #">
    # ---------------------------------------------------------------------------------------------
    def create_memoryview(self):
        '''
        BufferWrapper4Xi method create_memoryview returns address
        '''
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'BufferWrapper4Xi'
        method2print = 'create_memoryview'
        
        message2print = "{started" + "; create-memoryview-of-BufferWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 04:20:25 04/15/13 Pacific Daylight Time
                 
        if examine4xi(class2print, method2print):  
        
            # -------------------------------------------------------------------------------------    
            # StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) performing" + "; BufferWrapper-create_memoryview-performing-by-init-of-BufferWrapper4Xi: " + str(BufferWrapper.create_memoryview) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                
                             
            return BufferWrapper.create_memoryview(self)
            
        else:        
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------         
            (arena2xi, start2xi, stop2xi), size2xi = self._state
            
            
            message2print = "{set" + "; arena2xi-set-by-create-memoryview-of-BufferWrapper4Xi to: " + str(arena2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <arena4xi.Arena4Xi object at 0x0000000003A84F60>
            
            message2print = "{set" + "; start2xi-set-by-create-memoryview-of-BufferWrapper4Xi to: " + str(start2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 0  
            
            message2print = "{set" + "; stop2xi-set-by-create-memoryview-of-BufferWrapper4Xi to: " + str(stop2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 8
            
            message2print = "{set" + "; size2xi-set-by-create-memoryview-of-BufferWrapper4Xi to: " + str(size2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 4
                  
            message2print = "{returning" + "; memoryview-returning-by-create-memoryview-of-BufferWrapper4Xi: " + str(memoryview) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)                    
            # <class 'memoryview'>
            
            return memoryview(arena2xi.buffer)[start2xi:start2xi+size2xi]
       
    #</methodize4xi>  
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get-size-of-BufferWrapper4Xi" usage = "gets size of BufferWrapper4Xi">                   
    # ---------------------------------------------------------------------------------------------
    def get_size(self): 
        '''
        BufferWrapper4Xi method get_size returns size
        '''
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'BufferWrapper4Xi'
        method2print = 'get_size'
        
        
        message2print = "{started" + "; get-size-of-BufferWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
  
        if examine4xi(class2print, method2print):  
        
            # -------------------------------------------------------------------------------------    
            # StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) performing" + "; BufferWrapper-get_size-performing-by-init-of-BufferWrapper4Xi: " + str(BufferWrapper.get_size) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                
                             
            return BufferWrapper.get_size(self)
            
        else:            
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------     
            message2print = "{returning" + "; bufferwrapper2xi-state-returning-by-get-size-of-BufferWrapper4Xi: " + str(self._state[1]) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)   
            
            return self._state[1]
    #</methodize4xi>

              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-BufferWrapper4Xi" usage = "senses BufferWrapper4Xi class objects">
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses BufferWrapper4Xi class objects
        '''

        
        class2print = 'BufferWrapper4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-BufferWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
  

        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{sensing" + "; BufferWrapper4Xi-sensing-by-sense-of-BufferWrapper4Xi: " + str(BufferWrapper4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        kb2xi = sense_object4xi(self)
        
        try:
            kb2xi['inheritance-of-bufferwrapper2xi'] = object    
            kb2xi['heap-of-bufferwrapper2xi'] = str(BufferWrapper4Xi._heap)
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-BufferWrapper4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
                
        message2print = "{sensed" + "; BufferWrapper4Xi-sensed-by-sense-of-BufferWrapper4Xi: " + str(BufferWrapper4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-BufferWrapper4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)     
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-BufferWrapper4Xi" usage = "reasons BufferWrapper4Xi class objects">
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons BufferWrapper4Xi class objects
        '''
      
        
        class2print = 'BufferWrapper4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-BufferWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
       
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; BufferWrapper4Xi-reasoning-by-reason-of-BufferWrapper4Xi: "  + str(BufferWrapper4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        
        kb4xi = reason_object4xi(self)
        
        
        
        message2print = "{reasoned" + "; BufferWrapper4Xi-reasoned-by-reason-of-BufferWrapper4Xi: "  + str(BufferWrapper4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-BufferWrapper4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)       
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-BufferWrapper4Xi" usage = "aces BufferWrapper4Xi class objects">
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces BufferWrapper4Xi class objects 
        '''
       
        
        
        class2print = 'BufferWrapper4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-BufferWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
       
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; BufferWrapper4Xi-acing-by-ace-of-BufferWrapper4Xi: " + "BufferWrapper4Xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
      
        kb8xi = ace_object4xi(self)  
        
        message2print = "{aced" + "; BufferWrapper4Xi-aced-by-ace-of-BufferWrapper4Xi: " + "BufferWrapper4Xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-BufferWrapper4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)             
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------                
    #</methodize4xi>
#</class4xi>
#</program4xi> 

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

    
    

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of bufferwrapper4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

 